/* application.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "application.h"
#include "Components/RemoteControl/RemoteControl.h"

Application::Application(int & argc, char ** argv) :
	QApplication(argc, argv),
	SayonaraClass()
{
	_timer = new QTime();
	_timer->start();
}

void Application::check_for_crash(){

	QString error_file = Helper::get_error_file();

	if(!QFile::exists(error_file)) return;

	QString info_text;
	QString mail;

	if(!Helper::read_file_into_str(error_file, mail)){
		mail = "";
		mail.prepend("mailto:luciocarreras@gmail.com?subject=Sayonara Crash&amp;body=Hi Lucio,\n\nhere is the trace for a Sayonara crash\n\n");
	}

	else{
		mail.prepend("mailto:luciocarreras@gmail.com?subject=Sayonara Crash&amp;body=Hi Lucio,\n\nhere is the trace for a Sayonara crash\n\n");
		mail.append("\n\nI hope this will not happen again...");
	}

	info_text = QString("Sayonara seems to have crashed the last time<br />") +
			"Please send " +
			Helper::create_link(error_file, error_file) +
			" in " + Helper::create_link(Helper::get_sayonara_path(), Helper::get_sayonara_path()) +
			" to " + Helper::create_link("luciocarreras@gmail.com", mail);

	GlobalMessage::getInstance()->info(info_text);

	QFile f(error_file);
	f.open(QIODevice::ReadOnly);
	if(!f.isOpen()){
		sp_log(Log::Warning) << "Cannot oopen " << error_file;
		return;
	}

	f.remove();
	f.close();

	return;
}

bool Application::init(QTranslator* translator, const QStringList& files_to_play) {


	sp_log(Log::Debug) << "Init application: " << _timer->elapsed() << "ms";

	bool success = this->installTranslator(translator);
	if(!success){
		sp_log(Log::Warning) << "Cannot install translator";
	}

	//check_for_crash();

	QString version = QString(SAYONARA_VERSION);
	_settings->set(Set::Player_Version, version);


	GUI_Player* player	= new GUI_Player(translator);

	sp_log(Log::Debug) << "Init player: " << _timer->elapsed() << "ms";

#ifdef WITH_DBUS
	DBusHandler* dbus	= new DBusHandler(player, this);
	Q_UNUSED(dbus)

#endif

	RemoteControl* rmc = new RemoteControl(this);
	Q_UNUSED(rmc)

	if(_settings->get(Set::Notification_Show)){
		NotificationHandler::getInstance()->notify("Sayonara Player",
												   tr("Version ") + SAYONARA_VERSION,
												   Helper::get_share_path() + "logo.png");
	}

	sp_log(Log::Debug) << "Init plugins: " << _timer->elapsed() << "ms";
	PlayerPluginHandler* pph = new PlayerPluginHandler(this);

	pph->addPlugin( new GUI_LevelPainter());
	pph->addPlugin(new GUI_Spectrum());
	pph->addPlugin(new GUI_Equalizer());
	pph->addPlugin(new GUI_Stream());
	pph->addPlugin(new GUI_Podcasts());
	pph->addPlugin(new GUI_PlaylistChooser());
	pph->addPlugin(new GUI_AudioConverter());
	pph->addPlugin(new GUI_Bookmarks());
	pph->addPlugin(new GUI_Speed());
	pph->addPlugin(new GUI_Broadcast());

	sp_log(Log::Debug) << "Plugins finsihed: " << _timer->elapsed() << "ms";

	PlayManager::getInstance()->pause();

	QList<LibraryContainerInterface*> library_containers;
	DirectoryLibraryContainer* directory_container = new DirectoryLibraryContainer(this);
	LocalLibraryContainer* local_library_container = new LocalLibraryContainer(this);
	LibraryPluginHandler* library_plugin_loader = new LibraryPluginHandler(this);

	library_containers << static_cast<LibraryContainerInterface*>(local_library_container);
	library_containers << static_cast<LibraryContainerInterface*>(directory_container);
	library_plugin_loader->init(library_containers);

	sp_log(Log::Debug) << "Libraries loaded: " << _timer->elapsed() << "ms";


	player->register_preference_dialog(new GUI_LanguageChooser(player));
	player->register_preference_dialog(new GUI_FontConfig(player));
	player->register_preference_dialog(new GUI_StartupDialog(player));
	player->register_preference_dialog(new GUI_Notifications(player));
	player->register_preference_dialog(new GUI_LastFM(player));
	player->register_preference_dialog(new GUI_StreamRecorder(player));
	player->register_preference_dialog(new GUI_BroadcastSetup(player));
	player->register_preference_dialog(new GUI_RemoteControl(player));

	sp_log(Log::Debug) << "Preference dialogs loaded: " << _timer->elapsed() << "ms";

	player->setWindowTitle(QString("Sayonara ") + version);
	player->setWindowIcon(Helper::get_icon("logo.png"));
	player->set_libraries(library_plugin_loader);
	player->set_player_plugin_handler(pph);
	player->ui_loaded();

	if(_settings->get(Set::PL_StartPlaying)){
		PlayManager::getInstance()->play();
	}

	else{
		PlayManager::getInstance()->pause();
	}

	if(files_to_play.size() > 0) {
		PlaylistHandler* ph = PlaylistHandler::getInstance();
		QString playlist_name = ph->request_new_playlist_name();
		ph->create_playlist(files_to_play, playlist_name);
	}

	sp_log(Log::Debug) << "Time to start: " << _timer->elapsed() << "ms";


	return true;
}

Application::~Application() {

	delete _timer;

	DatabaseConnector::getInstance()->store_settings();
	DatabaseConnector::getInstance()->close_db();
}

