/* GSTPipeline.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "AbstractPipeline.h"
#include "Callbacks/EngineCallbacks.h"

#include <gst/app/gstappsink.h>

AbstractPipeline::AbstractPipeline(QString name, Engine* engine, QObject* parent) :
	QObject(parent),
	SayonaraClass()
{
	_initialized = false;
	_engine = engine;
	_name = name;
    _position_ms = 0;
    _duration_ms = 0;

	_about_to_finish = false;
}

AbstractPipeline::~AbstractPipeline(){
	if (_bus){
		gst_object_unref (_bus);
	}

	if (_pipeline) {
		gst_element_set_state(GST_ELEMENT(_pipeline), GST_STATE_NULL);
		gst_object_unref (GST_OBJECT(_pipeline));
	}
}


bool AbstractPipeline::init(GstState state){

	bool success = false;
	if(_initialized) {
		return true;
	}

	// create equalizer element
	_pipeline = gst_pipeline_new(_name.toStdString().c_str());
	if(!_test_and_error(_pipeline, "Engine: Pipeline sucks")){
		return false;
	}

	_bus = gst_pipeline_get_bus(GST_PIPELINE(_pipeline));

	success = create_elements();
	if(!success) {
		return false;
	}

	success = add_and_link_elements();
	if(!success) {
		return false;
	}

	configure_elements();

	gst_element_set_state(_pipeline, state);
	gst_bus_add_watch(_bus, EngineCallbacks::bus_state_changed, _engine);

	_initialized = true;
	return true;
}

void AbstractPipeline::refresh_position() {

	gint64 pos;
	bool success;

	success = gst_element_query_position(GST_ELEMENT(_pipeline), GST_FORMAT_TIME, &pos);

	if(success){
		_position_ms = GST_TIME_AS_MSECONDS(pos);
		//if(_duration_ms >= 0){
			emit sig_pos_changed_ms( _position_ms );
		//}
	}

	else {
		_position_ms = -1;
	}
}

void AbstractPipeline::refresh_duration(){

	gint64 dur;
	bool success;

	success = gst_element_query_duration(GST_ELEMENT(_pipeline), GST_FORMAT_TIME, &dur);
	if(success){
		_duration_ms = GST_TIME_AS_MSECONDS(dur);
	}

	refresh_position();
}

void AbstractPipeline::set_data(uchar* data, quint64 size){
	emit sig_data(data, size);
}

void AbstractPipeline::check_about_to_finish(){

	gint64 difference = _duration_ms - _position_ms;


	if(difference <= 0 && !_about_to_finish){
		refresh_duration();
		difference = _duration_ms - _position_ms;
		if(difference < 0){
			stop();
			return;
		}
	}


	/*sp_log(Log::Debug) << "Difference: "
					   << difference<< ": "
					   << _position_ms / 1000 << " - "
					   << _duration_ms / 1000;*/

	if(difference < 1000 && !_about_to_finish) {

		//sp_log(Log::Debug) << "About to finishe in " << difference;
		_about_to_finish = true;
		emit sig_about_to_finish((difference * 3) / 4);
	}

	else if(difference > 1000){
		_about_to_finish = false;
	}
}

qint64 AbstractPipeline::get_duration_ms(){
    return _duration_ms;
}

qint64 AbstractPipeline::get_position_ms(){
    return _position_ms;
}


void AbstractPipeline::set_speed(float f) {
	Q_UNUSED(f);
}

void AbstractPipeline::finished() {

	emit sig_finished();
}


GstBus* AbstractPipeline::get_bus() {
	return _bus;
}


GstState AbstractPipeline::get_state() {
	GstState state;
	gst_element_get_state(_pipeline, &state, nullptr, GST_MSECOND * 10);
	return state;
}


GstElement* AbstractPipeline::get_pipeline() {
	return _pipeline;
}

bool AbstractPipeline::set_uri(gchar* uri) {
	_uri = uri;
	return (_uri != nullptr);
}

gchar* AbstractPipeline::get_uri() {
	return _uri;
}


bool AbstractPipeline::create_element(GstElement** elem, const gchar* elem_name, const gchar* name){

	QString error_msg;
	if(strlen(name) > 0){
		*elem = gst_element_factory_make(elem_name, name);
		error_msg = QString("Engine: ") + name + " creation failed";
	}

	else{
		*elem = gst_element_factory_make(elem_name, elem_name);
		error_msg = QString("Engine: ") + elem_name + " creation failed";
	}

	return _test_and_error(*elem, error_msg);
}



bool
_test_and_error(void* element, QString errorstr) {

	if(!element) {
		sp_log(Log::Error) << errorstr;
		return false;
	}

	return true;
}

bool
_test_and_error_bool(bool b, QString errorstr) {
	if(!b) {
		sp_log(Log::Error) << errorstr;
		return false;
	}

	return true;
}
