/* PlaylistDBConnector.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "PlaylistDBConnector.h"
#include "Helper/Tagging/Tagging.h"
#include "Helper/Helper.h"
#include "Helper/Parser/PlaylistParser.h"

PlaylistDBConnector::PlaylistDBConnector()
{
	_db = DatabaseConnector::getInstance();



}


PlaylistDBConnector::~PlaylistDBConnector(){

}


void PlaylistDBConnector::apply_tags(MetaDataList& v_md){
	for(MetaData& md : v_md){
		if(md.is_extern){

			if(Helper::is_file(md.filepath())){
				Tagging::getMetaDataOfFile(md);
			}
		}
	}
}


bool PlaylistDBConnector::get_skeletons(QList<CustomPlaylistSkeleton>& skeletons, DatabasePlaylist::PlaylistChooserType type, SortOrderPlaylists so){
	return _db->getAllPlaylistSkeletons(skeletons, type, so);
}

bool PlaylistDBConnector::get_all_skeletons(QList<CustomPlaylistSkeleton>& skeletons,
					   SortOrderPlaylists so)
{
	return get_skeletons(skeletons,
						 DatabaseConnector::PlaylistChooserType::TemporaryAndNoTemporary,
						 so);
}

bool PlaylistDBConnector::get_temporary_skeletons(QList<CustomPlaylistSkeleton>& skeletons,
					   SortOrderPlaylists so)
{
	return get_skeletons(skeletons,
						 DatabaseConnector::PlaylistChooserType::OnlyTemporary,
						 so);
}

bool PlaylistDBConnector:: get_non_temporary_skeletons(QList<CustomPlaylistSkeleton>& skeletons,
					   SortOrderPlaylists so)
{
	return get_skeletons(skeletons,
						 DatabaseConnector::PlaylistChooserType::OnlyNoTemporary,
						 so);
}


bool PlaylistDBConnector::extract_stream(CustomPlaylist& pl, QString name, QString url){

	pl.is_temporary = false;

	if(Helper::is_playlistfile(url)) {
		if(PlaylistParser::parse_playlist(url, pl.tracks) > 0) {

			for(MetaData& md : pl.tracks) {

				md.album = name;
				if(md.title.isEmpty()){
					md.title = name;
				}

				if(md.artist.isEmpty()){
					md.artist = url;
				}

				md.is_extern = true;
				md.id = -1;
			}
		}
	}

	// real stream
	else{

		MetaData md;

		md.title = name;
		md.album = name;
		md.artist = url;
		md.set_filepath(url);

		md.is_extern = true;
		md.id = -1;

		pl.tracks << std::move(md);
	}

	pl.is_valid = (pl.tracks.size() > 0);

	return pl.is_valid;
}


bool PlaylistDBConnector::get_playlists(QList<CustomPlaylist>& playlists, DatabasePlaylist::PlaylistChooserType type, SortOrderPlaylists so){

	Q_UNUSED(type)

	bool success;
	QList<CustomPlaylistSkeleton> skeletons;

	success = get_all_skeletons(skeletons, so);
	if(!success){
		return false;
	}

	QMap<QString, QString> streams;
	success = _db->getAllStreams(streams);

	for(const CustomPlaylistSkeleton& skeleton : skeletons){

		CustomPlaylist pl(skeleton);

		bool is_stream = false;

		if(success){
			for(const QString& key : streams.keys()){
				if(skeleton.name.compare(key) == 0){
					is_stream = true;
					extract_stream(pl, key, streams[key]);
					break;
				}
			}
		}

		if(!is_stream){
			success = _db->getPlaylistById(pl);

			if(!success){
				continue;
			}

			apply_tags(pl.tracks);
		}

		playlists.push_back(pl);
	}

	return true;
}


bool PlaylistDBConnector::get_all_playlists(QList<CustomPlaylist>& playlists, SortOrderPlaylists so){
	return get_playlists(playlists,
						 DatabaseConnector::PlaylistChooserType::TemporaryAndNoTemporary,
						 so);
}


bool PlaylistDBConnector::get_temporary_playlists(QList<CustomPlaylist>& playlists, SortOrderPlaylists so){
	return get_playlists(playlists,
						 DatabaseConnector::PlaylistChooserType::OnlyTemporary,
						 so);
}


bool PlaylistDBConnector::get_non_temporary_playlists(QList<CustomPlaylist>& playlists, SortOrderPlaylists so){
	return get_playlists(playlists,
						 DatabaseConnector::PlaylistChooserType::OnlyNoTemporary,
						 so);
}


CustomPlaylist PlaylistDBConnector::get_playlist_by_id(int id){

	bool success;
	CustomPlaylist pl;
	pl.id = id;

	success = _db->getPlaylistById(pl);
	pl.is_valid = success;

	if(!success){
		return pl;
	}

	return pl;
}


CustomPlaylist PlaylistDBConnector::get_playlist_by_name(const QString& name){


	int id = _db->getPlaylistIdByName(name);

	if(id < 0){
		CustomPlaylist pl;
		pl.id = -1;
		pl.is_valid = false;
		return pl;
	}

	return get_playlist_by_id(id);
}

bool PlaylistDBConnector::rename_playlist(int id, const QString& new_name){
	return _db->renamePlaylist(id, new_name);
}


bool PlaylistDBConnector::save_playlist_as(const MetaDataList& v_md, const QString& name){
	return _db->storePlaylist(v_md, name, false);
}

bool PlaylistDBConnector::save_playlist_temporary(const MetaDataList& v_md, const QString& name){
	return _db->storePlaylist(v_md, name, true);
}


bool PlaylistDBConnector::save_playlist(const CustomPlaylist& pl){
	return _db->storePlaylist(pl.tracks, pl.id, pl.is_temporary);
}


bool PlaylistDBConnector::save_playlist(const MetaDataList& v_md, int id){

	bool success;
	CustomPlaylistSkeleton skeleton;
	skeleton.id = id;
	success = _db->getPlaylistSkeletonById(skeleton);

	if(!success){
		return false;
	}

	return _db->storePlaylist(v_md, id, skeleton.is_temporary);
}


bool PlaylistDBConnector::save_playlist(const MetaDataList& v_md, const QString name){

	CustomPlaylistSkeleton skeleton;
	int id = _db->getPlaylistIdByName(name);
	if(id < 0){
		return false;
	}

	skeleton.id = id;
	_db->getPlaylistSkeletonById(skeleton);

	return _db->storePlaylist(v_md, id, skeleton.is_temporary );
}


bool PlaylistDBConnector::delete_playlist(int id){
	return _db->deletePlaylist(id);
}


bool PlaylistDBConnector::delete_playlist(const QString& name){
	int id = _db->getPlaylistIdByName(name);
	return _db->deletePlaylist(id);
}


bool PlaylistDBConnector::exists(const QString& name){
	int id = _db->getPlaylistIdByName(name);
	return (id >= 0);
}
