/* Playlist.cpp */

/* Copyright (C) 2011-2016 Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Created on: Apr 6, 2011
 *      Author: luke
 */

#include "PlaylistHandler.h"
#include "StdPlaylist.h"
#include "StreamPlaylist.h"


#include "Helper/DirectoryReader/DirectoryReader.h"
#include "Helper/Parser/PlaylistParser.h"
#include "Components/TagEdit/MetaDataChangeNotifier.h"

PlaylistHandler::PlaylistHandler(QObject * parent) :
	QObject (parent),
	SayonaraClass()
{

	MetaDataChangeNotifier* md_change_notifier = MetaDataChangeNotifier::getInstance();

	_db = DatabaseConnector::getInstance();
	_engine = EngineHandler::getInstance();
	_play_manager = PlayManager::getInstance();
	_playlist_db_connector = PlaylistDBConnector::getInstance();

	_cur_playlist_idx = -1;
	_active_playlist_idx = -1;

	connect(_play_manager,	&PlayManager::sig_playstate_changed, this, &PlaylistHandler::playstate_changed);
	connect(_play_manager, &PlayManager::sig_next, this, &PlaylistHandler::next);
	connect(_play_manager, &PlayManager::sig_previous, this, &PlaylistHandler::previous);

	connect(_engine, &EngineHandler::sig_md_changed, this, &PlaylistHandler::md_changed_single);
	connect(_engine, &EngineHandler::sig_dur_changed, this, &PlaylistHandler::md_changed_single);

	connect(md_change_notifier, &MetaDataChangeNotifier::sig_metadata_changed, this, &PlaylistHandler::md_changed);
}

PlaylistHandler::~PlaylistHandler() {

}


void PlaylistHandler::emit_cur_track_changed(){

	MetaData md;
	bool success;
	int cur_track_idx;
	int playlist_idx;
	int playlist_id;


	PlaylistPtr pl = get_active();

	success = pl->get_cur_track(md);
	cur_track_idx = pl->get_cur_track_idx();
	playlist_idx = pl->get_idx();

	if(!success || cur_track_idx == -1){
		_play_manager->change_track(MetaData(), -1);
		return;
	}

	playlist_id = pl->get_id();
	if(playlist_id == -1){
		pl->insert_temporary_into_db();
		playlist_id = pl->get_id();
	}

	_settings->set(Set::PL_LastPlaylist, playlist_id);

	_play_manager->change_track(md, cur_track_idx);

	emit sig_cur_track_idx_changed( cur_track_idx,	playlist_idx );
}


int PlaylistHandler::load_old_playlists(){

	int last_playlist_idx=0;
	int last_track_idx=0;
	PlaylistPtr playlist;

	PlaylistLoader loader;

	_active_playlist_idx = -1;
	_cur_playlist_idx = -1;

	sp_log(Log::Debug) << "Loading playlists...";

	// returns -1 if fail, else the correct idx
	last_playlist_idx = loader.get_last_playlist_idx();
	last_track_idx = loader.get_last_tracks_idx();

	// no playlists found
	if( loader.get_playlist_count() == 0 ){
		create_empty_playlist(request_new_playlist_name());
		change_current_playlist_index(0);
		set_active_idx(0);
		_play_manager->stop();
	}

	else{

		// add playlists
		for(const CustomPlaylist& pl : loader.get_playlists()){

			create_playlist(pl);
		}

		playlist = get_playlist(last_playlist_idx, nullptr);

		// playlist not found -> track not found
		if(playlist == nullptr){
			last_track_idx = 0;
			last_playlist_idx = 0;
		}

		else{
			int n_tracks = playlist->get_size();

			// track not found
			if( !between(last_track_idx, 0, n_tracks) )
			{
				last_track_idx = 0;
			}
		}

		change_current_playlist_index(last_playlist_idx);
		set_active_idx(last_playlist_idx);
		change_track(last_track_idx, last_playlist_idx);

		if(!_settings->get(Set::PL_StartPlaying)){
			_play_manager->pause();
		}

		else{
			_play_manager->play();
		}
	}

	REGISTER_LISTENER(Set::PL_Mode, _sl_playlist_mode_changed);
	return _playlists.size();
}


PlaylistPtr PlaylistHandler::new_playlist(Playlist::Type type, int playlist_idx, QString name) {

	switch(type) {

		case Playlist::Type::Stream:
			return PlaylistPtr(new StreamPlaylist(playlist_idx, name));

		case Playlist::Type::Std:
		default:
			return PlaylistPtr(new StdPlaylist(playlist_idx, name));
	}

	return nullptr;
}


int PlaylistHandler::add_new_playlist(const QString& name, bool temporary, Playlist::Type type){

	PlaylistPtr pl;
	int idx = exists(name);

	if(idx >= 0) {
		return idx;
	}

	pl = new_playlist(type, _playlists.size(), name);
	pl->set_temporary(temporary);

	_playlists.append(pl);
	idx = _playlists.size() - 1;

	return idx;
}


// create a playlist, where metadata is already available
int PlaylistHandler::create_playlist(const MetaDataList& v_md, QString name, bool temporary, Playlist::Type type) {

	int idx;
	PlaylistPtr current;

	idx = exists(name);

	if(idx == -1){
		PlaylistPtr pl;

		idx = add_new_playlist(name, temporary, type);
		pl =  _playlists[idx];
		pl->insert_temporary_into_db();

		emit sig_new_playlist_added(pl);
	}

	_cur_playlist_idx = idx;

	change_current_playlist_index(idx);

	current = get_current();
	current->create_playlist(v_md);
	current->set_temporary( current->is_temporary() && temporary ) ;

	return current->get_idx();
}


// create a new playlist, where only filepaths are given
// Load Folder, Load File...
int PlaylistHandler::create_playlist(const QStringList& pathlist, QString name, bool temporary, Playlist::Type type) {

	DirectoryReader reader;
	MetaDataList v_md = reader.get_md_from_filelist(pathlist);
	return create_playlist(v_md, name, temporary, type);
}


int PlaylistHandler::create_playlist(const QString& dir, QString name, bool temporary, Playlist::Type type) {

	QStringList lst;
	lst << dir;

	return create_playlist(lst, name, temporary, type);
}


int PlaylistHandler::create_playlist(const CustomPlaylist& cpl){

	int idx=-1;

	// look if this playlist is already available
	for(const PlaylistPtr pl : _playlists){

		if(pl->get_id() == cpl.id){

			idx = pl->get_idx();
			pl->set_changed(false);

			break;
		}
	}

	// not available
	if(idx == -1){
		idx = add_new_playlist(cpl.name, cpl.is_temporary, Playlist::Type::Std);
		emit sig_new_playlist_added( _playlists[idx] );
	}

	PlaylistPtr pl;

	change_current_playlist_index( idx );

	pl = get_current();

	pl->enable_saving(false);
	pl->create_playlist(cpl.tracks);
	pl->enable_saving(true);

	return idx;
}


int PlaylistHandler::create_empty_playlist(const QString& name){

	MetaDataList v_md;
	return create_playlist(v_md, name, true);
}



void PlaylistHandler::save_playlist_to_file(const QString& filename, bool relative) {

	PlaylistParser::save_playlist(filename, get_current()->get_playlist(), relative);
}


void PlaylistHandler::reset_playlist(int idx){

	CustomPlaylist cpl;
	PlaylistPtr pl = get_playlist(idx, get_current());
	int id = pl->get_id();

	cpl = _playlist_db_connector->get_playlist_by_id(id);
	create_playlist(cpl);
}

PlaylistDBInterface::SaveAsAnswer PlaylistHandler::save_cur_playlist(){
	return save_playlist(_cur_playlist_idx);
}

PlaylistDBInterface::SaveAsAnswer PlaylistHandler::save_playlist(int idx){

	PlaylistDBInterface::SaveAsAnswer ret;

	if( !between(idx, 0, _playlists.size()) ){
		return PlaylistDBInterface::SaveAsAnswer::Error;
	}

	PlaylistPtr pl = _playlists[idx];
	ret = pl->save();

	if(!pl->is_temporary()){
		emit sig_saved_playlists_changed();
	}

	return ret;
}


PlaylistDBInterface::SaveAsAnswer PlaylistHandler::save_cur_playlist_as(const QString& name, bool force_override){

	return save_playlist_as(_cur_playlist_idx, name, force_override);
}

PlaylistDBInterface::SaveAsAnswer PlaylistHandler::save_playlist_as(int idx, const QString& name, bool force_override){

	PlaylistPtr pl;
	PlaylistDBInterface::SaveAsAnswer ret;

	// no empty playlists
	if(name.isEmpty()){
		return PlaylistDBInterface::SaveAsAnswer::Error;
	}

	// get playlist we wanna save
	pl = get_playlist(idx, nullptr);
	if( pl == nullptr ){
		return PlaylistDBInterface::SaveAsAnswer::Error;
	}

	ret = pl->save_as(name, force_override);
	if(ret != PlaylistDBInterface::SaveAsAnswer::Success){
		return ret;
	}

	// for PlaylistChooser
	if(!pl->is_temporary()){
		emit sig_saved_playlists_changed();
	}

	// for GUI
	emit sig_playlist_name_changed(idx);

	return PlaylistDBInterface::SaveAsAnswer::Success;
}

PlaylistDBInterface::SaveAsAnswer PlaylistHandler::rename_playlist(int idx, const QString& name){

	PlaylistPtr pl;
	PlaylistDBInterface::SaveAsAnswer ret;

	// no empty playlists
	if(name.isEmpty()){
		return PlaylistDBInterface::SaveAsAnswer::Error;
	}

	// get playlist we wanna save
	pl = get_playlist(idx, nullptr);
	if( pl == nullptr ){
		return PlaylistDBInterface::SaveAsAnswer::Error;
	}

	ret = pl->rename(name);
	if(ret != PlaylistDBInterface::SaveAsAnswer::Success){
		return ret;
	}

	emit sig_playlist_name_changed(idx);

	// for PlaylistChooser
	if(!pl->is_temporary()){
		emit sig_saved_playlists_changed();
	}

	return PlaylistDBInterface::SaveAsAnswer::Success;
}

void PlaylistHandler::delete_playlist(int idx){

	PlaylistPtr pl = _playlists[idx];
	bool was_temporary = pl->is_temporary();
	bool success = pl->remove_from_db();

	// for PlaylistChooser
	if(success && !was_temporary){
		emit sig_saved_playlists_changed();
	}
}


void PlaylistHandler::md_changed_single(const MetaData& md) {

	for( PlaylistPtr& pl : _playlists ){
		IdxList idx;
		idx << pl->find_tracks(md.filepath());

		for(int i : idx){
			pl->replace_track(i, md);
		}
	}
}


void PlaylistHandler::md_changed(const MetaDataList& v_md_old, const MetaDataList& v_md_new) {
	for(PlaylistPtr& pl : _playlists){
		pl->metadata_changed(v_md_old, v_md_new);
	}
}


// GUI -->
void PlaylistHandler::clear_current_playlist() {

	PlaylistPtr current = get_current();
	current->clear();

}

void PlaylistHandler::_sl_playlist_mode_changed(){

	PlaylistMode mode = _settings->get(Set::PL_Mode);
	for(PlaylistPtr& pl : _playlists){
		pl->set_playlist_mode(mode);
	}

}

void PlaylistHandler::playstate_changed(PlayManager::PlayState state){

	switch(state){
		case PlayManager::PlayState::Playing:
			played();
			break;
		case PlayManager::PlayState::Paused:
			paused();
			break;
		case PlayManager::PlayState::Stopped:
			stopped();
			break;

		default:
			return;
	}
}

void PlaylistHandler::played(){

	get_active()->play();
}

void PlaylistHandler::paused(){

	get_active()->pause();
}

void PlaylistHandler::stopped(){

	get_active()->stop();
}


void PlaylistHandler::next() {
	get_active()->next();
	emit_cur_track_changed();
}


void PlaylistHandler::previous() {
	get_active()->bwd();
	emit_cur_track_changed();
}


void PlaylistHandler::change_track(int idx, int playlist_idx) {

	if(_active_playlist_idx == -1){
		_active_playlist_idx = playlist_idx;
	}

	if(playlist_idx != _active_playlist_idx && playlist_idx >= 0){
		get_active()->stop();
		set_active_idx(playlist_idx);
	}

	get_active()->change_track(idx);

	emit_cur_track_changed();
}


void PlaylistHandler::set_active_idx(int idx){
	PlaylistPtr active;

	_active_playlist_idx = idx;
	active = get_active();

	_active_playlist_idx = active->get_idx();

	_settings->set(Set::PL_LastPlaylist, get_active()->get_id());
}


void PlaylistHandler::play_next(const MetaDataList& v_md) {

	PlaylistPtr active = get_active();

	active->insert_tracks(v_md, active->get_cur_track_idx() + 1);
}


void PlaylistHandler::insert_tracks(const MetaDataList& v_md, int row, int pl_idx) {

	PlaylistPtr pl = get_playlist(pl_idx, get_current());

	pl->insert_tracks(v_md, row);
}


void PlaylistHandler::append_tracks(const MetaDataList& v_md, int pl_idx) {

	PlaylistPtr pl = get_playlist(pl_idx, get_current());

	pl->append_tracks(v_md);

}

void PlaylistHandler::remove_rows(const IdxList& rows, int pl_idx) {

	PlaylistPtr pl = get_playlist(pl_idx, get_current());

	pl->delete_tracks(rows);
}

void PlaylistHandler::move_rows(const IdxList& rows, int tgt_idx, int pl_idx) {

	PlaylistPtr pl = get_playlist(pl_idx, get_current());

	pl->move_tracks(rows, tgt_idx);
}


QString PlaylistHandler::request_new_playlist_name() const
{
	return PlaylistDBInterface::request_new_db_name();
}



int PlaylistHandler::change_current_playlist_index(int idx){

	PlayManager::PlayState state;
	_cur_playlist_idx = idx;

	state = _play_manager->get_play_state();
	if(state == PlayManager::PlayState::Stopped){
		set_active_idx(idx);
	}

	return _cur_playlist_idx;
}

void PlaylistHandler::close_playlist(int idx){

	if(!between(idx, 0, _playlists.size())){
		return;
	}

	bool was_active = (idx == _active_playlist_idx);
	bool was_current = (idx == _cur_playlist_idx);

	if(_playlists[idx]->is_temporary()){
		QString name = _playlists[idx]->get_name();
		_playlist_db_connector->delete_playlist( name );
	}

	PlaylistPtr p_to_delete = _playlists[idx];
	_playlists.removeAt(idx);

	sp_log(Log::Debug) << "Refcount = " << p_to_delete.use_count();


	if( was_current ){
		_cur_playlist_idx = 0;
	}

	else if(_cur_playlist_idx > idx){
		_cur_playlist_idx --;
	}

	if( was_active ){
		set_active_idx(0);
	}

	else if(_active_playlist_idx > idx){
		_active_playlist_idx --;
	}

	for(PlaylistPtr pl : _playlists){
		if(pl->get_idx() >= idx){
			pl->set_idx(pl->get_idx() - 1);
		}
	}

	if(was_active){
		_settings->set(Set::PL_LastPlaylist, -1);
		_settings->set(Set::PL_LastTrack, -1);
	}
	else{
		_settings->set(Set::PL_LastPlaylist, get_active()->get_id());
	}
}

PlaylistConstPtr PlaylistHandler::get_playlist_at(int idx) const {

	if(! between(idx, 0, _playlists.size()) ){
		return nullptr;
	}

	return std::const_pointer_cast<const Playlist>(_playlists[idx]);
}

PlaylistPtr PlaylistHandler::get_playlist(int idx, PlaylistPtr fallback) const
{
	if(! between(idx, 0, _playlists.size())){
		return fallback;
	}

	return _playlists[idx];
}

PlaylistPtr PlaylistHandler::get_active()
{
	// assure we have at least one playlist
	if(_playlists.size() == 0){
		_active_playlist_idx = add_new_playlist(request_new_playlist_name(), true, Playlist::Type::Std);
		_cur_playlist_idx = _active_playlist_idx;
	}
	// assure valid idx
	if( !between(_active_playlist_idx, 0,_playlists.size()) ){
		_active_playlist_idx = 0;
	}

	return get_playlist(_active_playlist_idx, nullptr);
}

PlaylistPtr PlaylistHandler::get_current()
{

	// assure we have at least one playlist
	if(_playlists.size() == 0){
		_cur_playlist_idx = add_new_playlist(request_new_playlist_name(), true, Playlist::Type::Std);
		_active_playlist_idx = _cur_playlist_idx;
	}
	// assure valid idx
	if( !between(_cur_playlist_idx, 0, _playlists.size())){
		_cur_playlist_idx = 0;
	}

	return get_playlist(_cur_playlist_idx, nullptr);
}

int	PlaylistHandler::get_active_idx() const
{
	return _active_playlist_idx;
}

int	PlaylistHandler::get_current_idx() const
{
	return _cur_playlist_idx;
}

int PlaylistHandler::get_playlist_count() const
{
	return _playlists.size();
}

QStringList PlaylistHandler::get_playlist_names() const
{
	QStringList names;
	for(const PlaylistPtr& pl : _playlists){
		names << pl->get_name();
	}

	return names;
}

int PlaylistHandler::exists(const QString& name) const
{
	if(name.isEmpty()) {
		return _cur_playlist_idx;
	}

	for(const PlaylistPtr& pl : _playlists){

		if(pl->get_name().compare(name, Qt::CaseInsensitive) == 0){
			return pl->get_idx();
		}
	}

	return -1;
}
