/* GUI_Player.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "GUI_Player.h"
#include "GUI_TrayIcon.h"
#include "GUI/AlternativeCovers/GUI_AlternativeCovers.h"

#include "Helper/globals.h"
#include "Helper/Style/Style.h"
#include "Helper/WebAccess/AsyncWebAccess.h"

#include "Components/CoverLookup/CoverLookup.h"

#include <QFileDialog>
#include <QPalette>

GUI_Player* obj_ref = nullptr;

#ifdef Q_OS_UNIX
#include <csignal>

void signal_handler(int sig) {

	if(sig == SIGWINCH && obj_ref) {
		obj_ref->setHidden(false);
		obj_ref->show_again();
		obj_ref->activateWindow();
		obj_ref->read_filelist();
	}

	else if(sig == SIGTERM && obj_ref) {
		sp_log(Log::Debug) << "extern close event";
		obj_ref->really_close();
	}
	else {

		sp_log(Log::Debug) << "signal " << sig << " received";
	}
}

#endif


GUI_Player::GUI_Player(QTranslator* translator, QWidget *parent) :
	SayonaraMainWindow(parent),
	GlobalMessageReceiver("Player Main Window"),
	Ui::Sayonara()
{
	setupUi(this);

	GlobalMessage::getInstance()->register_receiver(this);

	init_gui();

	_translator = translator;
	_engine = EngineHandler::getInstance();
	_play_manager = PlayManager::getInstance();
	_local_library = LocalLibrary::getInstance();

	set_cur_pos_ms(_play_manager->get_init_position_ms());

	QString version = _settings->get(Set::Player_Version);

	_awa_version = new AsyncWebAccess(this);
	_awa_translators = new AsyncWebAccess(this);

#ifdef WITH_MTP
	_mtp = new GUI_MTP(this);
	action_devices->setVisible(true);
#else
	action_devices->setVisible(false);
#endif

	lab_sayonara->setText(tr("Sayonara Player"));
	lab_version->setText( version );
	lab_writtenby->setText(tr("Written by") + " Lucio Carreras");
	lab_copyright->setText(tr("Copyright") + " 2011-" + QString::number(QDateTime::currentDateTime().date().year()));

	progress_widget->setCurrentIndex(0);

	_skin_suffix = "";

	_cov_lookup = new CoverLookup(this);
	_ui_alternative_covers = new GUI_AlternativeCovers(this->centralWidget());

	action_min2tray->setChecked(_settings->get(Set::Player_Min2Tray));

	init_action(action_viewLibrary, Set::Lib_Show);
	init_action(action_livesearch, Set::Lib_LiveSearch);
	init_action(action_notifyNewVersion,Set::Player_NotifyNewVersion);
	init_action(action_Dark, Set::Player_Style);

	bool show_library = _settings->get(Set::Lib_Show);
	this->show_library(show_library);

	int volume = _play_manager->get_volume();
	volume_changed(volume);

	bool muted = _play_manager->get_mute();
	mute_changed(muted);

	skin_changed();

	bool is_fullscreen = _settings->get(Set::Player_Fullscreen);
	bool is_maximized = _settings->get(Set::Player_Maximized);
	if(!is_fullscreen && !is_maximized) {

		QSize size = _settings->get(Set::Player_Size);
		QPoint pos = _settings->get(Set::Player_Pos);
		QRect rect = this->geometry();

		rect.setX(pos.x());
		rect.setY(pos.y());
		rect.setWidth(size.width());
		rect.setHeight(size.height());
		this->setGeometry(rect);
	}

	/* tray actions */
	setup_tray_actions();

	// signals and slots
	setup_connections();

	plugin_widget->resize(plugin_widget->width(), 0);

	stopped();


	REGISTER_LISTENER(Set::Engine_SR_Active, _sl_sr_active_changed);

	REGISTER_LISTENER_NO_CALL(Set::Player_Fullscreen, _sl_fullscreen_toggled);
	REGISTER_LISTENER_NO_CALL(SetNoDB::Player_Quit, really_close);

	REGISTER_LISTENER(Set::Player_FontName, skin_changed);
	REGISTER_LISTENER(Set::Player_FontSize, skin_changed);
}


GUI_Player::~GUI_Player() {
	sp_log(Log::Info) << "closing player...";
}


void GUI_Player::language_changed() {

	QString language = _settings->get(Set::Player_Language);
	_translator->load(language, Helper::get_share_path() + "translations/");

	retranslateUi(this);
}


QAction* GUI_Player::create_actions(QList<QKeySequence>& seq_list) {
	QAction* action = new QAction(this);

	action->setShortcuts(seq_list);
	action->setShortcutContext(Qt::ApplicationShortcut);
	this->addAction(action);

	return action;
}

QAction* GUI_Player::create_action(QKeySequence seq) {
	QList<QKeySequence> seq_list;
	seq_list << seq;
	return create_actions(seq_list);
}


void GUI_Player::init_gui() {

	btn_mute->setIcon(Helper::get_icon("vol_1"));

	action_viewLibrary->setText(tr("&Library"));
	btn_rec->setVisible(false);

	action_Fullscreen->setShortcut(QKeySequence("F11"));
	action_Dark->setShortcut(QKeySequence("F10"));
}


// new track
void GUI_Player::track_changed(const MetaData & md) {

	_md = md;

	lab_sayonara->hide();
	lab_title->show();

	lab_version->hide();
	lab_artist->show();

	lab_writtenby->hide();
	lab_album->show();

	lab_copyright->hide();
	lab_rating->show();

	set_info_labels();

	set_total_time_label(_md.length_ms);

	QString rating_text;


	if(_md.bitrate / 1000 > 0){
		rating_text = QString::number(_md.bitrate / 1000) + " kBit/s";
	}

	if(_md.filesize > 0){
		if(_md.bitrate / 1000 > 0){
			rating_text += ", ";
		}
		rating_text += QString::number( (double) (_md.filesize / 1024) / 1024.0, 'f', 2) + " MB";
	}

	lab_rating->setText(rating_text);
	lab_rating->setToolTip(rating_text);

	set_radio_mode( _md.radio_mode() );

	this->setWindowTitle(QString("Sayonara - ") + md.title);
	this->repaint();

	fetch_cover();
}

void GUI_Player::set_title_label(){

	QString text = Helper::elide_text(_md.title, lab_title, 2);
	lab_title->setText(text);
}

void GUI_Player::set_info_labels()
{
	set_title_label();
	set_album_label();
	set_artist_label();
}

void GUI_Player::set_artist_label(){

	QFontMetrics fm = lab_artist->fontMetrics();
	lab_artist->setText( fm.elidedText(_md.artist, Qt::ElideRight, lab_artist->width()) );
}

void GUI_Player::set_album_label(){

	QString str_year = QString::number(_md.year);
	QString album_name = _md.album;
	QFontMetrics fm = lab_album->fontMetrics();

	if(_md.year > 1000 && !_md.album.contains(str_year)){
		album_name += " (" + str_year + ")";
	}

	lab_album->setText( fm.elidedText(album_name, Qt::ElideRight, lab_album->width()) );
}

void GUI_Player::dur_changed(const MetaData& md) {

	if(_md != md){
		return;
	}

	set_total_time_label(md.length_ms);
}

void GUI_Player::br_changed(const MetaData& md) {

	QString rating_text;

	if(_md != md){
		return;
	}

	if(md.bitrate / 1000 > 0){
		rating_text = QString::number(md.bitrate / 1000) + " kBit/s";
	}

	if(md.filesize > 0){
		if(md.bitrate / 1000 > 0){
			rating_text += ", ";
		}
		rating_text += QString::number( (double) (md.filesize / 1024) / 1024.0, 'f', 2) + " MB";
	}

	lab_rating->setText(rating_text);
	lab_rating->setToolTip(rating_text);

}

void GUI_Player::md_changed(const MetaData& md) {

	if(_md != md && md.title == _md.title){
		return;
	}

	sp_log(Log::Debug) << "Metadata changed ";
	md.print();

	_md = md;
	_md.title = md.title;

	lab_title->setText( md.title );
}


// public slot:
// id3 tags have changed
void GUI_Player::id3_tags_changed(const MetaDataList& v_md_old, const MetaDataList& v_md_new) {

	IdxList idxs = v_md_old.findTracks(_md.filepath());
	if(idxs.isEmpty()) return;

	int idx = idxs[0];

	_md = v_md_new[idx];

	set_info_labels();

	setWindowTitle(QString("Sayonara - ") + _md.title);
	fetch_cover();
}


void GUI_Player::skin_toggled(bool on){

	_settings->set(Set::Player_Style, (on ? 1 : 0) );
}

void GUI_Player::skin_changed() {

	bool dark = (_settings->get(Set::Player_Style) == 1);
	QString font_family = _settings->get(Set::Player_FontName);
	int font_size = _settings->get(Set::Player_FontSize);
	QString stylesheet = Style::get_style(dark, font_family, font_size);

	this->setStyleSheet(stylesheet);

	if (dark) {
		_skin_suffix = QString("_dark");
	}

	else {
		_skin_suffix = QString("");
	}

	setup_volume_button(sli_volume->value());
}

void GUI_Player::reload_skin(){
	skin_changed();
}

/** TRAY ICON **/
void GUI_Player::setup_tray_actions() {

	_tray_icon = new GUI_TrayIcon(this);
	_tray_icon->installEventFilter(this);

	connect(_tray_icon, &GUI_TrayIcon::sig_close_clicked, this, &GUI_Player::really_close);
	connect(_tray_icon, &GUI_TrayIcon::sig_show_clicked, this, &GUI_Player::show_again);
	connect(_tray_icon, &GUI_TrayIcon::sig_wheel_changed, this, &GUI_Player::change_volume_by_tick);
	connect(_tray_icon, &GUI_TrayIcon::activated, this, &GUI_Player::tray_icon_activated);

	_tray_icon->show();
}


void GUI_Player::tray_icon_activated (QSystemTrayIcon::ActivationReason reason) {

	bool min_to_tray = _settings->get(Set::Player_Min2Tray);
	switch (reason) {

	case QSystemTrayIcon::Trigger:

		if( this->isMinimized() ||
				 !this->isVisible() ||
				 !this->isActiveWindow())

		{
			show_again();
		}

		else{
			if(min_to_tray){
				this->setHidden(true);
			}

			else{
				this->showMinimized();
			}
		}

		break;

	case QSystemTrayIcon::MiddleClick:
		break;

	default:
		break;
	}
}


void GUI_Player::set_libraries(LibraryPluginHandler* plugin_loader) {

	_lph = plugin_loader;

	plugin_loader->set_library_parent(library_widget);
}

void GUI_Player::set_player_plugin_handler(PlayerPluginHandler* pph) {

	_pph = pph;

	QList<PlayerPluginInterface*> lst = _pph->get_all_plugins();
	QList<QAction*> actions;

	int i=1;
	for(PlayerPluginInterface* p : lst) {
		QAction* action = p->get_action();
		QKeySequence ks("Ctrl+F" + QString::number(i));
		action->setShortcut(ks);
		action->setData(p->get_name());
		// action is connected in Plugin itself
		p->setParent(plugin_widget);
		actions << action;
		i++;
	}

	menuView->insertActions(action_Dark, actions);
	menuView->insertSeparator(action_Dark);

	connect(_pph, &PlayerPluginHandler::sig_show_plugin, this, &GUI_Player::show_plugin);
	connect(_pph, &PlayerPluginHandler::sig_hide_all_plugins, this, &GUI_Player::hide_all_plugins);
}


/** LIBRARY AND PLAYLIST END **/


void GUI_Player::register_preference_dialog(PreferenceDialogInterface* dialog){

	for(QAction* action : menuPreferences->actions()){
		if(action->isSeparator()){
				menuPreferences->insertAction(action, dialog->get_action());
				break;
		}
	}
}

// prvt fct
void GUI_Player::set_radio_mode(RadioMode radio) {

	bool stream_ripper = _settings->get(Set::Engine_SR_Active);

	if(stream_ripper) {

		bool btn_rec_visible = (radio != RadioMode::Off);

		if(btn_rec_visible) {
			btn_play->setVisible(radio == RadioMode::Off);
			btn_rec->setVisible(radio != RadioMode::Off);
		}

		else {
			btn_rec->setVisible(radio != RadioMode::Off);
			btn_play->setVisible(radio == RadioMode::Off);
		}

		btn_play->setEnabled(radio == RadioMode::Off);
	}

	else{

		btn_rec->setVisible(false);
		btn_play->setVisible(true);
	}

	_tray_icon->set_enable_fwd(true);
	sli_progress->setEnabled( (_md.length_ms / 1000) > 0 );

	_play_manager->record(btn_rec->isChecked() && btn_rec->isVisible());

}


// public slot
void GUI_Player::_sl_sr_active_changed() {

	bool active = _settings->get(Set::Engine_SR_Active);

	if(active) {
		btn_play->setVisible(_md.radio_mode() == RadioMode::Off);
		btn_rec->setVisible(_md.radio_mode() != RadioMode::Off);
	}

	else{
		btn_play->setVisible(true);
		btn_play->setEnabled(_md.radio_mode() == RadioMode::Off);
		btn_rec->setVisible(false);
	}

	btn_rec->setChecked(false);
}



void GUI_Player::ui_loaded() {

#ifdef Q_OS_UNIX
	obj_ref = this;
	signal(SIGWINCH, signal_handler);
#endif

	bool fullscreen = _settings->get(Set::Player_Fullscreen);
	bool maximized = _settings->get(Set::Player_Maximized);
	QPoint pos = _settings->get(Set::Player_Pos);
	QSize sz = _settings->get(Set::Player_Size);

	action_Fullscreen->setChecked(fullscreen);

	this->setGeometry(pos.x(), pos.y(), sz.width(), sz.height());

	if(fullscreen){
		this->showFullScreen();
	}

	else if(maximized){
		this->showMaximized();
	}

	else{
		this->showNormal();
	}


	QString shown_plugin = _settings->get(Set::Player_ShownPlugin);
	if(!shown_plugin.isEmpty()){
		PlayerPluginInterface* p  = _pph->find_plugin(shown_plugin);
		show_plugin(p);
	}

	if(_play_manager->get_play_state() != PlayManager::PlayState::Stopped){
		MetaData md = _play_manager->get_cur_track();
		track_changed(md);
	}

	if(_settings->get(Set::Player_NotifyNewVersion)){
		_awa_version->run("http://sayonara-player.com/current_version");
	}

	QWidget* cur_library=nullptr;

	LibraryContainerInterface* cur_interface;
	cur_interface = _lph->get_cur_library();

	splitter->restoreState(_settings->get(Set::Player_SplitterState));
	splitter->layout()->update();
	splitter->update();

	if(cur_interface){
		cur_library = cur_interface->get_ui();

		if(cur_library == nullptr) return;

		cur_library->show();
		cur_library->resize(library_widget->size());
	}
}



void GUI_Player::awa_version_finished(bool success) {

	if(!success) return;

	QString new_version(_awa_version->get_data());
	QString cur_version = _settings->get(Set::Player_Version);
	bool notify_new_version = _settings->get(Set::Player_NotifyNewVersion);
	bool dark = (_settings->get(Set::Player_Style) == 1);

	new_version = new_version.trimmed();

	sp_log(Log::Info) << "Newest Version: " << new_version;
	sp_log(Log::Info) << "This Version:   " << cur_version;

	QString link;
	LINK("http://sayonara-player.com", "http://sayonara-player.com", dark, link);

	if(new_version > cur_version && notify_new_version) {
		Message::info(tr("A new version is available!") + "<br />" +  link);
	}
}

void GUI_Player::awa_translators_finished(bool success) {

	if(!success) return;

	QString data(_awa_translators->get_data());
	QStringList translators = data.split('\n');

	_translators.clear();

	for(const QString& str : translators) {
		if(str.trimmed().size() > 0) {
			_translators.push_back(str);
		}
	}

	about();
}

void GUI_Player::notify_new_version_toggled(bool b) {
	_settings->set(Set::Player_NotifyNewVersion, b);
}

void GUI_Player::read_filelist(){

	bool success;
	QStringList files_to_play;
	QString content;
	PlaylistHandler* playlist_handler;
	QString playlist_name;


	success = Helper::read_file_into_str("/tmp/sayonara_files", content);
	if(!success){
		return;
	}

	files_to_play = content.split("\n");
	if(files_to_play.isEmpty()){
		return;
	}

	QFile f("/tmp/sayonara_files");
	f.remove();

	playlist_handler = PlaylistHandler::getInstance();
	playlist_name = playlist_handler->request_new_playlist_name();
	playlist_handler->create_playlist(files_to_play,
									  playlist_name,
									  true,
									  Playlist::Type::Std
	);
}

void GUI_Player::show_again(){
	if(this->isHidden())
	{
		this->setHidden(false);
	}

	if(!this->isVisible()){
		this->setVisible(true);
	}

	this->setWindowFlags((Qt::WindowFlags) (windowFlags() & ~Qt::WindowMinimized));
	this->activateWindow();
	this->show();
	this->raise();
}

void GUI_Player::really_close() {

	this->hide();
	this->close();
}
