/* GUI_PlaylistChooser.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "GUI_PlaylistChooser.h"
#include "GUI/Helper/ComboBoxDelegate/ComboBoxDelegate.h"

GUI_PlaylistChooser::GUI_PlaylistChooser(QWidget *parent) :
	PlayerPluginInterface(parent),
	Ui::GUI_PlaylistChooser()
{
	_playlist_chooser = new PlaylistChooser();
}

GUI_PlaylistChooser::~GUI_PlaylistChooser() {
	delete _playlist_chooser;
}

QString GUI_PlaylistChooser::get_name() const
{
	return "Playlists";
}

QString GUI_PlaylistChooser::get_display_name() const
{
	return tr("Playlists");
}

QLabel* GUI_PlaylistChooser::get_title_label() const
{
	return lab_title;
}

QPushButton* GUI_PlaylistChooser::get_close_button() const
{
	return btn_close;
}


void GUI_PlaylistChooser::language_changed() {
	if(!is_ui_initialized()){
		return;
	}

	retranslateUi(this);
}

void GUI_PlaylistChooser::init_ui()
{

	if(is_ui_initialized()){
		return;
	}

	setup_parent(this);

	_last_dir = _settings->get(Set::Lib_Path);
	_cur_idx = -1;
	_text_before_save = "";
	_target_playlist_dialog = new GUI_Target_Playlist_Dialog(this);

	combo_playlistchooser->setItemDelegate(new ComboBoxDelegate(this));

	btn_tool_pl->show_delete(false);
	btn_tool_pl->show_save(false);

	btn_tool_file->show_save_as(true);
	btn_tool_file->show_open(true);

	connect(btn_tool_pl, &MenuToolButton::sig_save, this, &GUI_PlaylistChooser::save_button_pressed);
	connect(btn_tool_pl, &MenuToolButton::sig_delete, this, &GUI_PlaylistChooser::delete_button_pressed);

	connect(btn_tool_file, &MenuToolButton::sig_save_as, this, &GUI_PlaylistChooser::save_as_button_pressed);
	connect(btn_tool_file, &MenuToolButton::sig_open, this, &GUI_PlaylistChooser::load_button_pressed);

	connect(combo_playlistchooser, &QComboBox::editTextChanged, this, &GUI_PlaylistChooser::text_changed);
	connect(combo_playlistchooser, combo_activated_int, this, &GUI_PlaylistChooser::playlist_selected);


	connect(_target_playlist_dialog, &GUI_Target_Playlist_Dialog::sig_target_chosen,
			this, &GUI_PlaylistChooser::got_save_params);

	connect(_playlist_chooser, &PlaylistChooser::sig_all_playlists_loaded,
			this, &GUI_PlaylistChooser::all_playlists_fetched);

	_playlist_chooser->load_all_playlists();

}


void GUI_PlaylistChooser::all_playlists_fetched(const QList<CustomPlaylistSkeleton>& skeletons) {

	if(!is_ui_initialized()){
		return;
	}

	int n_entries;
    int tmp_cur_idx = _cur_idx;

	combo_playlistchooser->clear();
	combo_playlistchooser->addItem("", -1);

	for(const CustomPlaylistSkeleton& skeleton : skeletons)
	{
		combo_playlistchooser->addItem(skeleton.name, skeleton.id);
	}

    _cur_idx = tmp_cur_idx;
	n_entries = combo_playlistchooser->count();

	if( between(_cur_idx, 0, n_entries) ){
		combo_playlistchooser->setCurrentIndex(_cur_idx);
	}

	if( !_text_before_save.isEmpty() ){
		int idx = combo_playlistchooser->findText(_text_before_save);
		if( between(idx, 1, n_entries) ){
			combo_playlistchooser->setCurrentIndex(idx);
            _cur_idx = idx;
        }
    }

	text_changed(combo_playlistchooser->currentText());
}




void GUI_PlaylistChooser::save_button_pressed() {

	if( !between(_cur_idx, 0, combo_playlistchooser->count()) ){
		return;
	}

	QStringList lst;

	_text_before_save = combo_playlistchooser->currentText();

	for(int i=0; i<combo_playlistchooser->count(); i++) {

		QString txt = combo_playlistchooser->itemText(i);

		if(txt.size() > 0){
			lst << txt.toLower();
		}
    }

	if( lst.contains(_text_before_save.toLower()) ) {

		GlobalMessage::Answer answer = Message::question_yn(tr("Overwrite?"));

		if(answer == GlobalMessage::Answer::Yes){
			_playlist_chooser->save_playlist(_text_before_save);
		}

	}

	else if( !_text_before_save.isEmpty() ) {
		_playlist_chooser->save_playlist(_text_before_save);
    }

	else {
		return;
	}
}

void GUI_PlaylistChooser::got_save_params(const QString& filename, bool relative) {
	_playlist_chooser->save_playlist_file(filename, relative);
}

void GUI_PlaylistChooser::save_as_button_pressed() {

    _target_playlist_dialog->show();
}


void GUI_PlaylistChooser::delete_button_pressed() {

    _text_before_save = "";
	GlobalMessage::Answer answer = Message::question_yn(tr("Delete?"));

	if(_cur_idx < combo_playlistchooser->count() && _cur_idx != -1) {
		int val = combo_playlistchooser->itemData(_cur_idx).toInt();

		if(val >= 0 && answer == GlobalMessage::Answer::Yes){
			_playlist_chooser->delete_playlist(val);
		}
	}
}


void GUI_PlaylistChooser::playlist_selected(int idx) {

	_cur_idx = idx;
	if(_cur_idx >= combo_playlistchooser->count() || _cur_idx < 0) return;

	QString name = combo_playlistchooser->currentText();

	int val = combo_playlistchooser->itemData(idx).toInt();
    bool val_bigger_zero = (val > 0);
	btn_tool_pl->show_delete(val_bigger_zero);
	text_changed(name);

	_playlist_chooser->load_single_playlist(val);

	le_playlist_file->clear();
}


void GUI_PlaylistChooser::load_button_pressed() {

    QStringList filelist = QFileDialog::getOpenFileNames(
                    this,
                    tr("Open Playlist files"),
                    _last_dir,
                    Helper::get_playlistfile_extensions().join(" "));

    QString lab_text = "";
	for(const QString& filename : filelist) {

		int last_index_of_sep = filename.lastIndexOf(QDir::separator());
        _last_dir = filename.left(last_index_of_sep);

        QString trimmed_filename = filename.right(filename.size() - last_index_of_sep - 1);
        trimmed_filename = trimmed_filename.left(trimmed_filename.lastIndexOf('.'));
        lab_text += trimmed_filename + ", ";
    }

    if(lab_text.size() > 2)
        lab_text = lab_text.left(lab_text.size() - 2);

	le_playlist_file->setText(lab_text);

    if(filelist.size() > 0) {
		_playlist_chooser->playlist_files_selected(filelist);
    }
}


void GUI_PlaylistChooser::text_changed(const QString & text) {

	btn_tool_pl->show_save(text.size() > 0);

    QStringList lst;
	for(int i=0; i<combo_playlistchooser->count(); i++) {
		QString txt = combo_playlistchooser->itemText(i);
		if(txt.size() > 0){
			lst << txt.toLower();
		}
    }

	btn_tool_pl->show_delete(lst.contains(text.toLower()));
}
