/* GUI_AbstractStream.cpp */

/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "GUI_AbstractStream.h"

#include "Helper/Helper.h"
#include "Helper/MetaData/MetaData.h"
#include "Helper/Parser/PlaylistParser.h"
#include "Helper/Parser/PodcastParser.h"

#include <QAbstractItemView>
#include <QCompleter>



GUI_AbstractStream::GUI_AbstractStream(AbstractStreamHandler* stream_handler, QWidget* parent) :
	PlayerPluginInterface(parent)
{
	_db = DatabaseConnector::getInstance();

	_stream_handler = stream_handler;

	connect(_stream_handler, &AbstractStreamHandler::sig_error, this, &GUI_AbstractStream::error);
	connect(_stream_handler, &AbstractStreamHandler::sig_data_available, this, &GUI_AbstractStream::data_available);
}

GUI_AbstractStream::~GUI_AbstractStream(){

}

void GUI_AbstractStream::init_connections(){

	_btn_tool->show_new(true);
	connect(_btn_play, &QPushButton::clicked, this, &GUI_AbstractStream::listen_clicked);
	connect(_btn_tool, &MenuToolButton::sig_save, this, &GUI_AbstractStream::save_clicked);
	connect(_btn_tool, &MenuToolButton::sig_delete, this, &GUI_AbstractStream::delete_clicked);
	connect(_btn_tool, &MenuToolButton::sig_new, this, &GUI_AbstractStream::new_clicked);
	connect(_combo_stream, combo_current_index_changed_int, this, &GUI_AbstractStream::combo_idx_changed);
	connect(_combo_stream, &QComboBox::editTextChanged, this, &GUI_AbstractStream::combo_text_changed);
	connect(_le_url, &QLineEdit::textEdited, this, &GUI_AbstractStream::url_text_changed);
}

void GUI_AbstractStream::init_streams(){

	StreamMap data;

	if( _stream_handler->get_all_streams(data) ){
		setup_stations(data);
	}
}


void GUI_AbstractStream::language_changed(){

}

void GUI_AbstractStream::init_ui()
{

}

void GUI_AbstractStream::error(){
	_btn_play->setDisabled(false);
	_lab_listen->setText(tr("Listen"));

	sp_log(Log::Info) << "Stream Handler error";
	Message::warning(tr("Cannot open stream") + "\n" + _le_url->text());

}

void GUI_AbstractStream::data_available(){

	_btn_play->setDisabled(false);
	_lab_listen->setText(tr("Listen"));
}



void GUI_AbstractStream::play(QString url, QString station_name){

	sp_log(Log::Info) << "Play stream: " << station_name;
	bool success = _stream_handler->parse_station(url, station_name);
	if(!success){
		sp_log(Log::Info) << "Stream Handler busy";
	}
}


void GUI_AbstractStream::listen_clicked(){
	QString url;
	QString name;

	if( _combo_stream->currentIndex() <= 0) {
		url = _le_url->text();
		name = _title_fallback_name;
	}

	else{
		url = _le_url->text();
		name = _combo_stream->currentText();
	}

	url = url.trimmed();
	if(url.size() > 5) {

		play(url, name);
		_btn_play->setDisabled(true);
		_lab_listen->setText(tr("Busy..."));
	}
}

void GUI_AbstractStream::combo_idx_changed(int idx){

	QString cur_station_name = _combo_stream->currentText();
	QString address = _stations[cur_station_name];
	if(address.size() > 0) {
		_le_url->setText(address);
	}

	if(idx == 0) {
		_le_url->setText("");
	}

	_btn_tool->show_delete(idx > 0);
	_btn_tool->show_save(false);

	bool listen_enabled = (_le_url->text().size() > 5);
	_btn_play->setEnabled(listen_enabled);
	_lab_listen->setEnabled(listen_enabled);
	_combo_stream->setToolTip(address);
}



void GUI_AbstractStream::combo_text_changed(const QString& text){

	bool name_there = false;

	for( int i=0; i<_combo_stream->count(); i++ ) {

		QString str = _combo_stream->itemText(i);

		if( str.compare(text) == 0) {
			name_there = true;
			break;
		}
	}

	_btn_tool->show_delete(name_there);
	_btn_tool->show_save(text.size() > 0);

	bool listen_enabled = (_le_url->text().size() > 5);
	_btn_play->setEnabled(listen_enabled);
	_lab_listen->setEnabled(listen_enabled);
	_combo_stream->setToolTip("");
}

void GUI_AbstractStream::url_text_changed(const QString& url){

	QString key = _stations.key(url.trimmed());

	if(! key.isEmpty() ) {

		int idx = _combo_stream->findText(key, Qt::MatchCaseSensitive);

		if(idx != -1) {
			_combo_stream->setCurrentIndex(idx);
			_btn_tool->show_save(false);
			_btn_tool->show_delete(true);
		}
	}

	// new address
	else{

		_btn_tool->show_delete(false);

		bool save_enabled =
				(_combo_stream->currentText().size() > 0) &&
				(_le_url->text().size() > 5);

		_btn_tool->show_save(save_enabled);
		bool listen_enabled = (_le_url->text().size() > 5);
		_btn_play->setEnabled(listen_enabled);
		_lab_listen->setEnabled(listen_enabled);

		if(_combo_stream->currentIndex() > 0){
			_combo_stream->setCurrentIndex(0);
			_combo_stream->setEditText(tr("new"));
		}
	}
}

GlobalMessage::Answer GUI_AbstractStream::show_delete_confirm_dialog(){

	QString cur_station_name = _combo_stream->currentText();
	return Message::question_yn(tr("Do you really want to delete %1").arg(cur_station_name));
}

void GUI_AbstractStream::new_clicked(){
	if(_combo_stream->count() > 0){
		_combo_stream->setCurrentIndex(0);
		_combo_stream->setItemText(0, "");
	}

	else{
		_combo_stream->addItem("");
		_combo_stream->setCurrentIndex(0);
	}

	_le_url->setText("");
}

void GUI_AbstractStream::delete_clicked(){

	if(_combo_stream->currentIndex() <= 0) return;

	QString cur_station_name = _combo_stream->currentText();
	GlobalMessage::Answer ret = show_delete_confirm_dialog();

	if(ret == GlobalMessage::Answer::Yes) {

		if( _stream_handler->delete_stream(cur_station_name) ) {

			StreamMap map;
			sp_log(Log::Info) << cur_station_name << "successfully deleted";

			if( _stream_handler->get_all_streams(map) ) {
				setup_stations(map);
			}
		}
	}
}

void GUI_AbstractStream::save_clicked(){

	QString name = _combo_stream->currentText();
	QString url = _le_url->text();

	bool success = false;
	if(name.size() > 0 && url.size() > 0) {
		success = _stream_handler->add_stream(name, url);
	}

	if(success) {

		StreamMap map;

		if( _stream_handler->get_all_streams(map)) {

			setup_stations(map);

			for(int i=0; i<_combo_stream->count(); i++){
				QString s = _combo_stream->itemText(i);
				if(s.compare(name) == 0){
					_combo_stream->setCurrentIndex(i);
					break;
				}
			}
		}
	}

	_le_url->setText(url);

	url_text_changed(url);
}

void GUI_AbstractStream::setup_stations(const StreamMap& stations){

	_combo_stream->clear();

	_stations = stations;
	_stations[""] = "";

	if(_stations.size() == 1){
		_combo_stream->setCurrentIndex(0);
	}

	for(auto it = _stations.begin(); it != _stations.end(); it++) {
		_combo_stream->addItem(it.key(), it.value());
	}

	_combo_stream->view()->setItemDelegate(new ComboBoxDelegate(this));

	_btn_play->setEnabled(false);
	_lab_listen->setEnabled(false);
	_btn_tool->show_save(false);
	_btn_tool->show_delete(false);
}

