
/* Copyright (C) 2011-2016  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "GUI_Playlist.h"


void GUI_Playlist::playlist_name_changed(int idx){

	PlaylistConstPtr pl = _playlist->get_playlist_at(idx);
	if(!pl){
		return;
	}

	bool temporary = pl->is_temporary();
	bool was_changed = pl->was_changed();
	QString name = pl->get_name();

	if(!temporary && was_changed){
		tw_playlists->setTabText(idx, QString("* ") + name);
	}
	else{
		tw_playlists->setTabText(idx, name);
	}

	for(int i = tw_playlists->count() - 1; i>=0; i--) {
		if(i == idx){
			continue;
		}

		if(tw_playlists->tabText(i).compare(name) == 0){
			tw_playlists->removeTab(i);
		}
	}
}

void GUI_Playlist::playlist_changed(int idx)
{
	PlaylistConstPtr pl = _playlist->get_playlist_at(idx);

	if(!pl->is_temporary()){
		tw_playlists->setTabText(idx, QString("* ") + pl->get_name());
	}

	if(idx == tw_playlists->currentIndex()){
		qint64 dur_ms = 0;
		const MetaDataList& v_md = pl->get_playlist();

		for(const MetaData& md : v_md) {
			dur_ms += md.length_ms;
		}

		_total_time[idx] = dur_ms;
		set_total_time_label(dur_ms);
	}
}


void GUI_Playlist::playlist_added(PlaylistPtr pl){

	PlaylistView* pl_view = new PlaylistView(pl);

	int idx = pl->get_idx();
	QString name = pl->get_name();
	QIcon icon_play;

	init_playlist_view(pl_view);

	tab_clear_all_icons();

	_playlist_views.append(pl_view);
	_total_time.append(0);

	tw_playlists->addTab(pl_view, name);
	tw_playlists->setCurrentIndex(idx);
	tw_playlists->setTabsClosable(_playlist_views.size() > 1);

	int active_idx = _playlist->get_active_idx();

	if(between(active_idx, 0, _playlist_views.size())){

		int n_rows = _playlist_views[active_idx]->get_num_rows();
		if(n_rows > 0){

			icon_play = Helper::get_icon("play_bordered");
			tw_playlists->setTabIcon(active_idx, icon_play);
		}
	}

	connect(pl.get(), &Playlist::sig_data_changed, this, &GUI_Playlist::playlist_changed);
}


void GUI_Playlist::playstate_changed(PlayManager::PlayState state){
	if(state == PlayManager::PlayState::Stopped){
		tab_clear_all_icons();
	}
}

void GUI_Playlist::playlist_finished(){
	tab_clear_all_icons();
}


/** GUI SLOTS **/

void GUI_Playlist::tab_playlist_clicked(int idx){

	const PlaylistConstPtr pl = _playlist->get_playlist_at(idx);
	if(!pl){
		return;
	}

	bool temporary = pl->is_temporary();
	bool was_changed = pl->was_changed();

	tw_playlists->setCurrentIndex(idx);
	tw_playlists->show_menu_items(!temporary,	// save
								  true,			// save_as
								  !temporary,	// delete
								  !temporary && was_changed);	// reset

	set_total_time_label( _total_time[idx] );

	_playlist->change_current_playlist_index(idx);
}

void GUI_Playlist::tab_close_playlist_clicked(int idx){

	if( !between(idx, 0, tw_playlists->count())) {
		return;
	}

	QIcon icon_play;

	QWidget* playlist_widget = tw_playlists->widget(idx);

	_playlist_views.removeAt(idx);
	_total_time.remove(idx);
	tw_playlists->removeTab(idx);
	tw_playlists->setTabsClosable(_playlist_views.size() > 1);

	tab_clear_all_icons();
	icon_play = Helper::get_icon("play_bordered");
	tw_playlists->setTabIcon(_playlist->get_active_idx(), icon_play);

	delete playlist_widget;
	_playlist->close_playlist(idx);
}


void GUI_Playlist::tab_reset_clicked(int idx){
	_playlist->reset_playlist(idx);
}


void GUI_Playlist::tab_save_playlist_clicked(int idx){
	sp_log(Log::Debug) << "GUI: save_playlist";
	PlaylistDBInterface::SaveAsAnswer success = _playlist->save_playlist(idx);

	if(success == PlaylistDBInterface::SaveAsAnswer::Success){
		QString old_string = tw_playlists->tabText(idx);
		old_string.replace("* ", "");
		tw_playlists->setTabText(idx, old_string);
	}

	show_save_message_box(success);
}

void GUI_Playlist::tab_save_playlist_as_clicked(int idx, const QString& str){


	PlaylistDBInterface::SaveAsAnswer success = _playlist->save_playlist_as(idx, str, false);
	if(success == PlaylistDBInterface::SaveAsAnswer::AlreadyThere){

		GlobalMessage::Answer answer = show_save_message_box(success);

		if(answer == GlobalMessage::Answer::No) {
			return;
		}

		else{
			success = _playlist->save_playlist_as(idx, str, true);
			show_save_message_box(success);
		}
	}

	else{
		show_save_message_box(success);
	}
}


void GUI_Playlist::tab_rename_clicked(int idx, const QString& str){


	PlaylistDBInterface::SaveAsAnswer success = _playlist->rename_playlist(idx, str);
	if(success == PlaylistDBInterface::SaveAsAnswer::AlreadyThere){
		Message::error(tr("Playlist name already exists"));
	}

	else{
		show_save_message_box(success);
	}
}


void GUI_Playlist::tab_delete_playlist_clicked(int idx){

	GlobalMessage::Answer answer = Message::question_yn(tr("Really"), tr("Delete"));

	if(answer == GlobalMessage::Answer::No) {
		return;
	}

	_playlist->delete_playlist(idx);
}



/** Private methods **/

GlobalMessage::Answer GUI_Playlist::show_save_message_box(PlaylistDBInterface::SaveAsAnswer answer){

	switch(answer){
		case PlaylistDBInterface::SaveAsAnswer::Error:
			Message::warning(tr("Cannot save playlist."), tr("Save as"));
			break;

		case PlaylistDBInterface::SaveAsAnswer::AlreadyThere:
			return Message::question_yn(tr("Playlist exists") + "\n" + tr("Overwrite?"), tr("Save as"));

		case PlaylistDBInterface::SaveAsAnswer::ExternTracksError:
			return Message::warning(tr("Playlists are currently only supported for library tracks."), tr("Save playlist"));

		default:
			return GlobalMessage::Answer::Undefined;
	}

	return GlobalMessage::Answer::Undefined;
}


void GUI_Playlist::tab_clear_all_icons(){

	QIcon icon;

	for(int i=0; i<tw_playlists->count(); i++){
		tw_playlists->setTabIcon(i, icon);
		tw_playlists->setIconSize(QSize(16, 16));
	}
}


PlaylistView* GUI_Playlist::get_view_by_idx(int idx){
	if(!between(idx, 0, _playlist_views.size())){
		return nullptr;
	}

	return _playlist_views[idx];
}


PlaylistView* GUI_Playlist::get_current_view(){

	int cur_idx = tw_playlists->currentIndex();
	return get_view_by_idx(cur_idx);
}
