/* PlaylistView.cpp */

/* Copyright (C) 2011-2016 Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * PlaylistView.cpp
 *
 *  Created on: Jun 26, 2011
 *      Author: luke
 */

#include "PlaylistView.h"
#include "Helper/DirectoryReader/DirectoryReader.h"
#include "Components/Playlist/PlaylistHandler.h"

#include <QApplication>

PlaylistView::PlaylistView(PlaylistPtr pl, QWidget* parent) :
	SearchableListView(parent)
{
	_model = new PlaylistItemModel(pl, this);
	_delegate = new PlaylistItemDelegate(this);

	_drag = nullptr;
	_inner_drag_drop = false;
	_saved_selection_before_removal = -1;

	init_rc_menu();

	this->setModel(_model);
	this->setAbstractModel(_model);
	this->setItemDelegate(_delegate);
	this->setDragEnabled(true);
	this->setDropIndicatorShown(true);
	this->setDragDropMode(QAbstractItemView::DragDrop);
	this->setAcceptDrops(true);
	this->setSelectionMode(QAbstractItemView::ExtendedSelection);
	this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

	this->setAlternatingRowColors(true);
	this->setMovement(QListView::Free);

	this->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

	connect(this, &PlaylistView::pressed, this, &PlaylistView::row_pressed);
	connect(this, &PlaylistView::doubleClicked, this, &PlaylistView::row_double_clicked);
	connect(this, &PlaylistView::clicked, this, &PlaylistView::row_released);
}

PlaylistView::~PlaylistView() {

	if(_rc_menu){
		delete _rc_menu;
	}

	delete _model;
}


void PlaylistView::mousePressEvent(QMouseEvent* event) {

	if(_model->rowCount() == 0){
		return;
	}

	QPoint pos_org;
	QPoint pos;
	int entry_mask;
	IdxList selections;

	switch (event->button()) {

	case Qt::LeftButton:

		SearchableListView::mousePressEvent(event);

		_drag_pos = event->pos();

		break;

	case Qt::RightButton:

		SearchableListView::mousePressEvent(event);

		pos_org = event->pos();
		pos = QWidget::mapToGlobal(pos_org);

		pos.setY(pos.y());
		pos.setX(pos.x() + 10);

		entry_mask = (ENTRY_INFO | ENTRY_REMOVE);


		selections = get_selections();
		if(selections.size() == 1){
			entry_mask |= ENTRY_LYRICS;
		}

		if(_model->has_local_media(selections) ){
			entry_mask |= ENTRY_EDIT;
		}

		set_context_menu_actions(entry_mask);

		_rc_menu->exec(pos);

		break;

	default:
		break;
	}
}


void PlaylistView::mouseMoveEvent(QMouseEvent* event) {

	int distance = (event->pos() - _drag_pos).manhattanLength();

	if( event->buttons() & Qt::LeftButton &&
			distance >= QApplication::startDragDistance())
	{
		CustomMimeData* mimedata = _model->get_custom_mimedata(this->selectedIndexes());

		if(_drag){
			delete _drag;
		}

		_drag = new QDrag(this);

		connect(_drag, &QDrag::destroyed, this, &PlaylistView::drag_deleted);

		_drag->setMimeData(mimedata);
		_drag->exec(Qt::CopyAction);
	}
}

void PlaylistView::drag_deleted(){
	_drag = nullptr;
}


void PlaylistView::mouseReleaseEvent(QMouseEvent* event) {

	switch (event->button()) {

	case Qt::LeftButton:

		SearchableListView::mouseReleaseEvent(event);
		event->accept();

		break;

	default:
		break;
	}
}


// mark row as currently pressed
void PlaylistView::goto_row(int row) {

	if( (row >= _model->rowCount()) || (row < 0) ) {
		return;
	}

	QModelIndex idx = _model->index(row, 0);
	row_released(idx);

	this->scrollTo(idx, SearchableListView::EnsureVisible);
}


void PlaylistView::keyPressEvent(QKeyEvent* event) {

	int key = event->key();

	if((key == Qt::Key_Up || key == Qt::Key_Down)) {
		if(this->selectionModel()->selection().isEmpty()) {
			if(_model->rowCount() > 0) select_row(0);
			return;
		}
	}

	Qt::KeyboardModifiers  modifiers = event->modifiers();
	SearchableListView::keyPressEvent(event);

	if(!event->isAccepted() ) return;

	int new_row = -1;
	int min_row = get_min_selected();

	switch(key) {
	case Qt::Key_A:
		if( modifiers & Qt::ControlModifier ) {
			select_all();
		}
		break;

	case Qt::Key_Delete:
		_model->remove_rows(get_selections());
		break;

	case Qt::Key_End:
		new_row = _model->rowCount() - 1;
		break;

	case Qt::Key_Home:
		new_row = 0;
		break;

	case Qt::Key_Return:
	case Qt::Key_Enter:
		_model->set_current_track(min_row);
		emit sig_double_clicked(min_row);

		break;

	default:
		break;
	}

	if(new_row != -1) {
		goto_row(new_row);
	}
}

void PlaylistView::resizeEvent(QResizeEvent *e) {

	SearchableListView::resizeEvent(e);

	this->set_delegate_max_width(_model->rowCount());
}

void PlaylistView::init_rc_menu() {

	_rc_menu = new ContextMenu(this);

	connect(_rc_menu, &ContextMenu::sig_info_clicked, this, &PlaylistView::sig_info_clicked);
	connect(_rc_menu, &ContextMenu::sig_lyrics_clicked, this, &PlaylistView::sig_lyrics_clicked);
	connect(_rc_menu, &ContextMenu::sig_edit_clicked, this, &PlaylistView::sig_edit_clicked);
	connect(_rc_menu, &ContextMenu::sig_remove_clicked, this, &PlaylistView::remove_cur_selected_rows);
}

void PlaylistView::set_context_menu_actions(int actions) {
	_rc_menu->setup_entries(actions);
}

int PlaylistView::get_num_rows() {
	return _model->rowCount();
}

void PlaylistView::set_current_track(int row) {

	QModelIndex new_idx = _model->index(row);
	scrollTo(new_idx, SearchableListView::EnsureVisible);
}


void PlaylistView::clear() {
	clear_selection();
	_model->clear();
}


void PlaylistView::fill(PlaylistPtr pl) {

	this->set_delegate_max_width(pl->get_size());
	clear_selection();


	for(int i=0; i<pl->get_size(); i++){

		if(pl->at_const_ref(i).pl_playing) {
			this->scrollTo(_model->index(i), SearchableListView::EnsureVisible);
			return;
		}
	}


}


void PlaylistView::row_pressed(const QModelIndex& idx) {
	Q_UNUSED(idx)
	_inner_drag_drop = true;
}

void PlaylistView::row_released(const QModelIndex& idx) {
	Q_UNUSED(idx)
	_inner_drag_drop = false;
}

void PlaylistView::row_double_clicked(const QModelIndex& idx) {
	_inner_drag_drop = false;

	if(idx.isValid()) {
		_model->set_current_track(idx.row());
	}

	emit sig_double_clicked(idx.row());
}


void PlaylistView::selectionChanged ( const QItemSelection& selected, const QItemSelection & deselected ) {

	SearchableListView::selectionChanged(selected, deselected);

	if(selected.indexes().size() > 0){
		scrollTo(selected.indexes()[0], SearchableListView::EnsureVisible);
	}


}


// remove the black line under the titles
void  PlaylistView::clear_drag_drop_lines(int row) {

	_delegate->set_drag_index(-1);
	this->update(_model->index(row));
}


int PlaylistView::calc_drag_drop_line(QPoint pos) {

	if(pos.y() < 0) {
		return -1;
	}

	int row = this->indexAt(pos).row();

	if(row <= -1) {
		row = _model->rowCount()-1;
	}

	return row;
}


// the drag comes, if there's data --> accept it
void PlaylistView::dragEnterEvent(QDragEnterEvent* event) {
	event->accept();
}

void PlaylistView::dragMoveEvent(QDragMoveEvent* event) {

	event->accept();

	int row = calc_drag_drop_line(event->pos() );
	bool is_old = _delegate->is_drag_index(row);

	if(!is_old){
		clear_drag_drop_lines(_delegate->get_drag_index());
		_delegate->set_drag_index(row);
		this->update(_model->index(row));
	}
}


// we start the drag action, all lines has to be cleared
void PlaylistView::dragLeaveEvent(QDragLeaveEvent* event) {

	event->accept();
	clear_drag_drop_lines(_delegate->get_drag_index());
}

// called from GUI_Playlist when data has not been dropped
// directly into the view widget. Insert on first row then
void PlaylistView::dropEventFromOutside(QDropEvent* event) {

	event->accept();
	handle_drop(event, true);
}


void PlaylistView::dropEvent(QDropEvent* event) {

	event->accept();
	handle_drop(event, false);
}


void PlaylistView::handle_drop(QDropEvent* event, bool from_outside) {

	MetaDataList v_md;
	QString text;
	IdxList cur_selected_rows, new_selected_rows;

	// where did i drop?
	int row = _delegate->get_drag_index();
	clear_drag_drop_lines(row);

	if(from_outside) {
		row = -1;
	}

	if(_inner_drag_drop) {
		_inner_drag_drop = false;
		cur_selected_rows = get_selections();

		if( cur_selected_rows.contains(row) ) {
			event->ignore();
			return;
		}

		for(int i=0; i<cur_selected_rows.size(); i++){
			new_selected_rows << row + 1 + i - cur_selected_rows.size();
		}

		_model->move_rows(cur_selected_rows, row + 1);

		this->select_rows( new_selected_rows );

		return;
	}

	const QMimeData* mimedata = event->mimeData();
	const CustomMimeData* custom_mimedata = dynamic_cast<const CustomMimeData*>(mimedata);
	if(!mimedata) {
		return;
	}

	if(mimedata->hasText()) {
		text = mimedata->text();
	}

	// extern
	if( mimedata->hasUrls() && text.compare("tracks", Qt::CaseInsensitive) != 0) {
		QStringList filelist;

		for(const QUrl& url : mimedata->urls()) {
			QString path;
			QString url_str = url.toString();
			path =  url_str.right(url_str.length() - 7).trimmed();
			path = path.replace("%20", " ");
			filelist << std::move(path);
		}

		DirectoryReader reader;
		reader.set_filter(Helper::get_soundfile_extensions());

		v_md = reader.get_md_from_filelist(filelist);

		if(v_md.isEmpty()) {
			return;
		}
	}

	else if(mimedata->hasHtml()) {

	}
	else if(mimedata->hasImage()) {

	}

	else if(custom_mimedata != nullptr){

		if(	custom_mimedata->hasText() &&
				custom_mimedata->hasMetaData())
		{
			int sz = custom_mimedata->getMetaData(v_md);
			if(sz == 0) {
				return;
			}
		}
	}

	_model->insert_metadata(v_md, row + 1);
}

void PlaylistView::scroll_up() {
	QPoint p(5, 5);
	int cur_row = this->indexAt(p).row();
	if(cur_row <= 0) {
		return;
	}

	this->scrollTo(_model->index(cur_row - 1), SearchableListView::EnsureVisible);
}

void PlaylistView::scroll_down() {
	QPoint p(5, this->y() + this->height() - 5);
	int cur_row = this->indexAt(p).row();
	if(cur_row <= 0) {
		return;
	}

	this->scrollTo(_model->index(cur_row - 1), SearchableListView::EnsureVisible);
}

void PlaylistView::remove_cur_selected_rows() {

	int row_count = _model->rowCount();

	_saved_selection_before_removal = get_min_selected();

	if(_saved_selection_before_removal == (row_count - 1) && row_count >= 1)
	{
		_saved_selection_before_removal -= 1;
	}

	_model->remove_rows(get_selections());
}


void PlaylistView::set_delegate_max_width(int n_items) {

	int row_height = _delegate->get_row_height();
	bool scrollbar_visible = (( n_items * row_height ) >= this->height());

	int max_width = this->width();
	if(scrollbar_visible){
		max_width -= verticalScrollBar()->width();
	}

	_delegate->set_max_width(max_width);
}
